//
//  LAActionTemplate.h
//  LAAdditions
//
//  Created by Cyril Anger on 15/04/12.
//  Copyright (c) 2012 LyricApps. All rights reserved.
//

#import <LAAdditions/LATemplate.h>
#import <LAAdditions/LARule.h>

@interface LAActionTemplate : LATemplate

- (LAAction *)createActionInDatabaseManager:(LADatabaseManager *)databaseManager;
- (void)applyAction:(LAAction *)action toObject:(id)object withContext:(NSMutableDictionary *)context;

@end

@interface LASetVarActionTemplate : LAActionTemplate

- (id)initWithName:(NSString *)name extractableKeyPaths:(NSArray *)extractableKeyPaths;
+ (NSArray *)types;
@property (strong, nonatomic, readonly) NSArray *extractableKeyPaths;

@end

@interface LAKeyPathActionTemplate : LAActionTemplate

- (id)initWithName:(NSString *)name keyPath:(NSString *)keyPath;
@property (strong, nonatomic, readonly) NSString *keyPath;

@end

@interface LASimpleActionTemplate : LAKeyPathActionTemplate

- (id)initWithName:(NSString *)name keyPath:(NSString *)keyPath settableKeyPaths:(NSArray *)settableKeyPaths;
@property (strong, nonatomic, readonly) NSArray *settableKeyPaths;
+ (BOOL)canParseVars;
+ (id)objectFromString:(NSString *)string;
@property (strong, nonatomic, readonly) NSArray *types;
+ (Class)parameterClass;
@property (nonatomic, readonly) NSString *widgetKey;
- (id)createWidgetForObject:(id)object;
- (void)updateWidget:(id)widget forObject:(id)object;
- (void)setValue:(id)value toObject:(id)object;

@end

@interface LABooleanActionTemplate : LASimpleActionTemplate

@end

@interface LAListActionTemplate : LASimpleActionTemplate

@end

@interface LAStringActionTemplate : LASimpleActionTemplate

@end

@interface LADateActionTemplate : LAStringActionTemplate

@end

@interface LANumberActionTemplate : LAStringActionTemplate

@end

@interface LAPercentageActionTemplate : LANumberActionTemplate

@end

@interface LAColorActionTemplate : LASimpleActionTemplate

@end

@interface LAStringReplacementActionTemplate : LAKeyPathActionTemplate

+ (NSArray *)types;

@end

@interface LASwapActionTemplate : LAActionTemplate

- (id)initWithName:(NSString *)name keyPathsGroups:(NSArray *)keyPathsGroups;
@property (strong, nonatomic, readonly) NSArray *keyPathsGroups;

@end
