//
//  LABezierPathAdditions.h
//  LAAdditions
//
//  Created by Cyril Anger on 23/02/05.
//  Copyright 2011 LyricApps. All rights reserved.
//

#import <Foundation/Foundation.h>
#if TARGET_OS_IPHONE
#import <UIKit/UIKit.h>
#else
#import <AppKit/AppKit.h>
#endif

#import <LAAdditions/LAStructures.h>
#import <LAAdditions/LAColorAdditions.h>

#if TARGET_OS_IPHONE
#define LABezierPath UIBezierPath
#define lineToPoint addLineToPoint
#define curveToPoint addCurveToPoint
#define appendBezierPath appendPath
#define LARectCorner UIRectCorner
#define LARectCornerTopLeft UIRectCornerTopLeft
#define LARectCornerTopRight UIRectCornerTopRight
#define LARectCornerBottomLeft UIRectCornerBottomLeft
#define LARectCornerBottomRight UIRectCornerBottomRight
#define LARectCornerAllCorners UIRectCornerAllCorners
#else
#define LABezierPath NSBezierPath
enum {
    LARectCornerTopLeft = 1 << 0,
    LARectCornerTopRight = 1 << 1,
    LARectCornerBottomLeft = 1 << 2,
    LARectCornerBottomRight = 1 << 3,
    LARectCornerAllCorners = ~0
};
typedef NSUInteger LARectCorner;
#endif

typedef enum {
	LATriangleOrientationUp = 0,
	LATriangleOrientationRight = 1,
	LATriangleOrientationDown = 2,
	LATriangleOrientationLeft = 3
} LATriangleOrientation;

@interface LAGradient : NSObject

@property (strong, nonatomic) LAColor *startColor;
@property (strong, nonatomic) LAColor *endColor;

+ (LAGradient *)gradientWithColor:(LAColor *)color;
+ (LAGradient *)transparentGradient;

- (id)initWithStartColor:(LAColor *)startColor endColor:(LAColor *)endColor;

@end

@interface LABezierPath (LABezierPathAdditions)

#if !TARGET_OS_IPHONE
@property (readonly) CGPathRef CGPath;
@property BOOL usesEvenOddFillRule;
#endif

+ (LABezierPath *)bezierPathWithLineFromPoint:(LAPoint)point1 toPoint:(LAPoint)point2;
#if !TARGET_OS_IPHONE
+ (LABezierPath *)bezierPathWithRoundedRect:(LARect)rect cornerRadius:(CGFloat)cornerRadius;
+ (LABezierPath *)bezierPathWithRoundedRect:(LARect)rect byRoundingCorners:(LARectCorner)corners cornerRadii:(LASize)cornerRadii;
- (void)addArcWithCenter:(LAPoint)center radius:(CGFloat)radius startAngle:(CGFloat)startAngle endAngle:(CGFloat)endAngle clockwise:(BOOL)clockwise;
- (void)fillWithBlendMode:(CGBlendMode)blendMode alpha:(CGFloat)alpha;
- (void)strokeWithBlendMode:(CGBlendMode)blendMode alpha:(CGFloat)alpha;
#endif
- (void)fillInnerShadowWithColor:(LAColor *)shadowColor offset:(LASize)shadowOffset blur:(CGFloat)shadowBlur;
- (void)fillGradient:(LAGradient *)gradient vertically:(BOOL)vertically;
+ (LABezierPath *)fillProgressGradientInRect:(LARect)rect withRoundedCorners:(LARectCorner)roundedCorners color:(LAColor *)color;
+ (LABezierPath *)bezierPathWithTriangleInRect:(LARect)rect orientation:(LATriangleOrientation)orientation;
- (void)appendBezierPathWithTriangleInRect:(LARect)rect orientation:(LATriangleOrientation)orientation;
- (void)translateXBy:(CGFloat)deltaX yBy:(CGFloat)deltaY;

@end
