//
//  LAColumn.h
//  LAAdditions
//
//  Created by Cyril Anger on 08/09/11.
//  Copyright (c) 2011 LyricApps. All rights reserved.
//

#import <Foundation/Foundation.h>

#import <LAAdditions/LAStringAdditions.h>
#import <LAAdditions/LAViewAdditions.h>

@class LAColumnsSet;

@interface LAColumn : NSObject <NSCoding>

@property (weak, nonatomic, readonly) LAColumnsSet *columnsSet;
@property (strong, nonatomic, readonly) NSString *name;
@property (readonly) NSString *title;
@property (readonly) NSString *menuTitle;
@property (readonly) LAImage *image;
@property (nonatomic, readonly) NSInteger tag;
@property (nonatomic, readonly) NSTextAlignment alignment;
@property (nonatomic, readonly) NSInteger resizePriority;
@property (nonatomic, readonly) CGFloat minWidth;
@property (nonatomic, readonly) CGFloat maxWidth;
@property (nonatomic) CGFloat width;
@property (getter = isEnabled, nonatomic) BOOL enabled;
@property (getter = isHidden, nonatomic) BOOL hidden;
@property (getter = isVisible, nonatomic, readonly) BOOL visible;
@property (strong, nonatomic) id representedObject;

- (id)initWithName:(NSString *)name tag:(NSInteger)tag alignment:(NSTextAlignment)alignment resizePriority:(NSInteger)resizePriority minWidth:(CGFloat)minWidth maxWidth:(CGFloat)maxWidth enabled:(BOOL)enabled hidden:(BOOL)hidden;

@end

@interface NSArray (LAColumnAdditions)

- (NSArray *)duplicatedColumns;

@end

@protocol LAColumnsSetDelegate;

@interface LAColumnsSet : NSObject <NSCoding, NSCopying>

@property (strong, nonatomic, readonly) NSString *name;
@property (strong, nonatomic, readonly) NSMutableArray *columns;
@property (nonatomic) CGFloat leftOffset;
@property (nonatomic) CGFloat rightOffset;
@property (nonatomic) CGFloat intercolumnSpacing;
@property (nonatomic, readonly) BOOL allowsCustomization;
@property (weak, nonatomic) LAView *parentView;
@property (weak, nonatomic) id <LAColumnsSetDelegate> delegate;
@property (nonatomic, readonly) CGFloat width;

- (id)initWithName:(NSString *)name columns:(NSArray *)columns leftOffset:(CGFloat)leftOffset rightOffset:(CGFloat)rightOffset intercolumnSpacing:(CGFloat)intercolumnSpacing allowsCustomization:(BOOL)allowsCustomization delegate:(id <LAColumnsSetDelegate>)delegate;
- (LAColumn *)columnNamed:(NSString *)name;
- (LAColumn *)columnWithTag:(NSInteger)tag;
- (LAColumn *)columnAtOffset:(CGFloat)offset;
- (void)insertColumn:(LAColumn *)column atIndex:(NSUInteger)index;
- (void)addColumn:(LAColumn *)column;
- (void)removeColumn:(LAColumn *)column;
- (void)replaceColumnAtIndex:(NSUInteger)index withColumn:(LAColumn *)column;
- (NSDictionary *)saveState;
- (void)restoreState:(NSDictionary *)state;
- (void)updateColumnsOrder:(NSArray *)sortedColumns;
- (BOOL)sizeToFitWidth:(CGFloat)width;
- (BOOL)sizeToFit;
- (CGFloat)maxOffsetForShrinkingFromColumn:(LAColumn *)column;
- (void)shrinkFromColumn:(LAColumn *)column withOffset:(CGFloat)offset;
- (CGFloat)maxOffsetForExpandingFromColumn:(LAColumn *)column;
- (void)expandFromColumn:(LAColumn *)column withOffset:(CGFloat)offset;
- (void)updateView:(LAView *)view withLeftOffset:(CGFloat)leftOffset;

@end

@protocol LAColumnsSetDelegate <NSObject>

@optional

- (void)columnsSetDidResize:(LAColumnsSet *)columnsSet;
- (void)columnsSet:(LAColumnsSet *)columnsSet columnWasClicked:(LAColumn *)column;

@end
