//
//  LACurrencyFormatter.h
//  LAAdditions
//
//  Created by Cyril Anger on 09/02/07.
//  Copyright 2011 LyricApps. All rights reserved.
//

#import <LAAdditions/LANumberFormatter.h>

#define LAConvert(amount, currency, newCurrency) (currency == newCurrency || !currency.changeRate || !newCurrency.changeRate || [newCurrency.changeRate isEqualToNumber:[NSDecimalNumber zero]] ? amount : [amount decimalNumberByMultiplyingBy:[currency.changeRate decimalNumberByDividingBy:newCurrency.changeRate] withCurrency:newCurrency])

@protocol LACurrency <NSObject>

@property (strong, nonatomic, readonly) NSString *code;
@property (strong, nonatomic, readonly) NSString *symbol;
@property (nonatomic, readonly) short fractionDigits;
@property (strong, nonatomic, readonly) id <NSDecimalNumberBehaviors> behavior;
@property (strong, nonatomic, readonly) NSDecimalNumber *changeRate;

@end

@interface LACurrency : NSObject <NSCoding, LACurrency>

@property (strong, nonatomic) NSString *code;
@property (strong, nonatomic) NSString *symbol;
@property (nonatomic) short fractionDigits;
@property (strong, nonatomic) NSDecimalNumber *changeRate;

+ (LACurrency *)defaultCurrency;

- (instancetype)initWithCode:(NSString *)code symbol:(NSString *)symbol fractionDigits:(short)fractionDigits changeRate:(NSDecimalNumber *)changeRate;

@end

@interface NSDecimalNumber (LACurrencyAdditions)

- (NSDecimalNumber *)decimalNumberByAdding:(NSDecimalNumber *)decimalNumber withCurrency:(id <LACurrency>)currency;
- (NSDecimalNumber *)decimalNumberBySubtracting:(NSDecimalNumber *)decimalNumber withCurrency:(id <LACurrency>)currency;
- (NSDecimalNumber *)decimalNumberByMultiplyingBy:(NSDecimalNumber *)decimalNumber withCurrency:(id <LACurrency>)currency;
- (NSDecimalNumber *)decimalNumberByDividingBy:(NSDecimalNumber *)decimalNumber withCurrency:(id <LACurrency>)currency;

@end

@interface LACurrencyFormatter : LANumberFormatter

@property (nonatomic) BOOL canInsertDecimalSeparatorAutomatically;
@property (strong, nonatomic) id <LACurrency> currency;
@property (strong, nonatomic) NSArray *currencies;

+ (NSString *)formatValue:(NSDecimalNumber *)value withCurrency:(id <LACurrency>)currency;

- (void)updateCurrency;

@end
