//
//  LADatabaseConnection.h
//  LAAdditions
//
//  Created by Cyril Anger on 14/01/2015.
//  Copyright (c) 2015 LyricApps. All rights reserved.
//

#import <Foundation/Foundation.h>

#import <LAAdditions/LADuration.h>
#import <LAAdditions/LAConstant.h>
#import <LAAdditions/LADatabaseManager.h>

@interface LADatabaseConnectionQuery : NSObject

- (void)reset;
- (void)setNullForParameter:(NSString *)parameter;
- (void)setBool:(BOOL)value forParameter:(NSString *)parameter;
- (void)setInt:(int)value forParameter:(NSString *)parameter;
- (void)setInt64:(long long)value forParameter:(NSString *)parameter;
- (void)setDouble:(double)value forParameter:(NSString *)parameter;
- (void)setString:(NSString *)value forParameter:(NSString *)parameter;
- (void)setData:(NSData *)value forParameter:(NSString *)parameter;
- (void)setDate:(NSDate *)value forParameter:(NSString *)parameter;
- (void)setDateTime:(NSDate *)value forParameter:(NSString *)parameter;
- (void)setDuration:(LADuration *)value forParameter:(NSString *)parameter;
- (void)setDecimalNumber:(NSDecimalNumber *)value forParameter:(NSString *)parameter;
- (void)setImage:(LAImage *)image forParameter:(NSString *)parameter;
- (void)setColor:(LAColor *)color forParameter:(NSString *)parameter;
- (void)setJSONObject:(id)object forParameter:(NSString *)parameter;
- (void)setConstant:(LAConstant *)constant forParameter:(NSString *)parameter;
- (void)setClass:(Class)class forParameter:(NSString *)parameter;

@end

@interface LADatabaseConnectionResultSet : NSObject

- (BOOL)next;
- (BOOL)isNullForColumn:(NSString *)column;
- (BOOL)boolForColumn:(NSString *)column;
- (int)intForColumn:(NSString *)column;
- (long long)int64ForColumn:(NSString *)column;
- (double)doubleForColumn:(NSString *)column;
- (NSString *)stringForColumn:(NSString *)column;
- (NSData *)dataForColumn:(NSString *)column;
- (NSDate *)dateForColumn:(NSString *)column;
- (NSDate *)dateTimeForColumn:(NSString *)column;
- (LADuration *)durationForColumn:(NSString *)column;
- (NSDecimalNumber *)decimalNumberForColumn:(NSString *)column;
- (LAImage *)imageForColumn:(NSString *)column;
- (LAColor *)colorForColumn:(NSString *)column;
- (id)JSONObjectForColumn:(NSString *)column;
- (id)constantForColumn:(NSString *)column;
- (Class)classForColumn:(NSString *)column;

@end

@interface LADatabaseConnection : NSObject

- (instancetype)init NS_UNAVAILABLE;
- (instancetype)initWithURL:(NSURL *)URL readOnly:(BOOL)readOnly deleteOnDealloc:(BOOL)deleteOnDealloc error:(NSError * __autoreleasing *)error NS_DESIGNATED_INITIALIZER;
@property (strong, nonatomic, readonly) NSURL *URL;
@property (getter = isReadOnly, nonatomic, readonly) BOOL readOnly;
@property (nonatomic, readonly) BOOL deleteOnDealloc;

- (LADatabaseConnectionQuery *)prepareQuery:(NSString *)sql error:(NSError * __autoreleasing *)error;
- (LADatabaseConnectionResultSet *)executeQuery:(LADatabaseConnectionQuery *)query error:(NSError * __autoreleasing *)error;
- (BOOL)executeUpdate:(LADatabaseConnectionQuery *)query error:(NSError * __autoreleasing *)error;

@end
