//
//  LADatabaseObject.h
//  LAAdditions
//
//  Created by Cyril Anger on 14/01/2015.
//  Copyright (c) 2015 LyricApps. All rights reserved.
//

#import <Foundation/Foundation.h>

#import <LAAdditions/LASynchronizedObject.h>
#import <LAAdditions/LADatabaseManager.h>

@interface LADatabaseObject : NSObject <LASynchronizedObject, NSCoding>

#pragma mark Initialization

- (instancetype)init NS_DESIGNATED_INITIALIZER;
- (instancetype)initWithDatabaseManager:(LADatabaseManager *)databaseManager;
- (instancetype)initWithDatabaseManager:(LADatabaseManager *)databaseManager row:(LADatabaseResultSet *)row;

#pragma mark Copy

- (instancetype)initWithDatabaseManager:(LADatabaseManager *)databaseManager byCopyingObject:(id)object;
- (instancetype)copyInDatabaseManager:(LADatabaseManager *)databaseManager;

#pragma mark Duplication

- (instancetype)duplicateInDatabaseManager:(LADatabaseManager *)databaseManager;

#pragma mark Registration

@property (weak, nonatomic, readonly) LADatabaseManager *databaseManager;
@property (atomic) long long sqlID;
@property (strong, nonatomic) NSString *ID;

#pragma mark Modifications handling

@property (nonatomic, readonly) NSInteger modificationsCount;
@property (strong, nonatomic) NSDate *lastModificationDate;
- (void)addModification;

#pragma mark Save

+ (NSString *)table;
+ (NSArray *)columns;
- (NSArray *)computeChanges;
@property (nonatomic, readonly) LADatabaseChangePrepareBlock prepareBlock;
- (void)commitChangesToOriginalDatabaseManager;

#pragma mark Deletion

@property (getter = isDeleted, nonatomic) BOOL deleted;
- (void)delete:(BOOL)registerDeletion;

#pragma mark Synchronization

+ (NSUInteger)synchronizationPriority;
@property (getter = isBeingImported, nonatomic, readonly) BOOL beingImported;
- (instancetype)initWithDatabaseManager:(LADatabaseManager *)databaseManager byImportingObject:(id)object;
- (instancetype)importInDatabaseManager:(LADatabaseManager *)databaseManager;
- (NSArray *)compareWithObject:(LADatabaseObject *)otherObject;

#pragma mark References

@property (strong, nonatomic, readonly) NSHashTable *referringParameters;

@end

@interface LASynchronizationChange (LADatabaseObjectAdditions)

+ (instancetype)compareReference:(LADatabaseObject *)value ofObject:(LADatabaseObject *)object withReference:(LADatabaseObject *)otherValue ofObject:(LADatabaseObject *)otherObject localizationKey:(NSString *)key block:(void (^)(id object, id value))block;
+ (instancetype)compareParent:(LADatabaseObject *)value ofObject:(LADatabaseObject *)object inArray:(NSArray *)array withParent:(LADatabaseObject *)otherValue ofObject:(LADatabaseObject *)otherObject inArray:(NSArray *)otherArray localizationKey:(NSString *)key block:(void (^)(id object, id parent, NSArray *IDs))block;
+ (NSArray *)compareOwnedReference:(LADatabaseObject *)value ofObject:(LADatabaseObject *)object withOwnedReference:(LADatabaseObject *)otherValue ofObject:(LADatabaseObject *)otherObject localizationKey:(NSString *)key block:(void (^)(id object, id value))block;
+ (instancetype)compareReferencesSet:(NSSet *)value ofObject:(LADatabaseObject *)object withReferencesSet:(NSSet *)otherValue ofObject:(LADatabaseObject *)otherObject localizationKey:(NSString *)key block:(void (^)(id object, NSMutableSet *set))block;

@end
