//
//  LADropboxManager.h
//  LAAdditions
//
//  Created by Cyril Anger on 01/03/2016.
//  Copyright © 2016 LyricApps. All rights reserved.
//

#import <LAAdditions/LAJSONConnection.h>
#import <LAAdditions/LAURLDownload.h>

extern NSString *LADropboxErrorDomain;

typedef enum {
	LADropboxErrorNone = 0,
	LADropboxErrorAPICallFailed
} LADropboxErrorCode;

@interface LADropboxEntry : NSObject

@property (strong, nonatomic, readonly) NSString *ID;
@property (strong, nonatomic, readonly) NSString *name;
@property (strong, nonatomic, readonly) NSString *path;

@end

@interface LADropboxFile : LADropboxEntry

@property (strong, nonatomic, readonly) NSDate *modificationDate;
@property (strong, nonatomic, readonly) NSString *revision;

@end

@interface LADropboxFolder : LADropboxEntry

@end

@interface LADropboxManager : NSObject

@property (strong, nonatomic) NSString *appKey;
@property (strong, nonatomic) NSString *secret;
@property (nonatomic, readonly) NSURL *loginURL;
@property (strong, nonatomic) NSString *token;
@property (strong, nonatomic) NSDate *tokenExpirationDate;
@property (strong, nonatomic) NSString *refreshToken;

+ (LADropboxManager *)sharedManager;

- (BOOL)handleOpenURL:(NSURL *)URL;
- (void)listFolder:(NSString *)folder completionHandler:(void (^)(NSArray *entries, NSError *error))completionHandler setCurrentConnectionBlock:(void (^)(LAURLConnection *connection))setCurrentConnectionBlock;
- (NSArray *)listFolder:(NSString *)folder error:(NSError * __autoreleasing *)error;
- (LADropboxEntry *)getEntry:(NSString *)path error:(NSError * __autoreleasing *)error;
- (LAURLDownload *)downloadFile:(NSString *)file toPath:(NSString *)path completionHandler:(void (^)(NSError *error))completionHandler;
- (LADropboxFile *)uploadURL:(NSURL *)URL toPath:(NSString *)path withModificationDate:(NSDate *)modificationDate currentRevision:(NSString *)revision error:(NSError * __autoreleasing *)error;
- (void)deleteFile:(LADropboxFile *)file completionHandler:(void (^)(NSError *error))completionHandler setCurrentConnectionBlock:(void (^)(LAURLConnection *connection))setCurrentConnectionBlock;

@end

@interface LADropboxFolderObserver : NSObject

@property (strong, nonatomic, readonly) NSString *path;

- (instancetype)initWithPath:(NSString *)path changesHandler:(void (^)(void))changesHandler;

@end
