//
//  LAImageAdditions.h
//  LAAdditions
//
//  Created by Cyril Anger on 18/11/04.
//  Copyright 2011 LyricApps. All rights reserved.
//

#import <LAAdditions/LABezierPathAdditions.h>

CGFloat LAGraphicsGetImageScale();
void LAGraphicsSetImageScale(CGFloat scale);
#if TARGET_OS_IPHONE
#define LAImage UIImage
#define LABeginImage(result, size) UIGraphicsBeginImageContextWithOptions(size, NO, LAGraphicsGetImageScale())
#define LAEndImage(result) result = UIGraphicsGetImageFromCurrentImageContext(); UIGraphicsEndImageContext()
#define LAImageJPEGRepresentation UIImageJPEGRepresentation
#define LAImagePNGRepresentation UIImagePNGRepresentation
#define LABlendMode CGBlendMode
#define LABlendModeClear kCGBlendModeClear
#define LABlendModeSourceOver kCGBlendModeNormal
#define LABlendModeSourceIn kCGBlendModeSourceIn
#define LABlendModeSourceOut kCGBlendModeSourceOut
#define LABlendModeSourceAtop kCGBlendModeSourceAtop
#define LABlendModeDestinationOver kCGBlendModeDestinationOver
#define LABlendModeDestinationIn kCGBlendModeDestinationIn
#define LABlendModeDestinationOut kCGBlendModeDestinationOut
#define LABlendModeDestinationAtop kCGBlendModeDestinationAtop
#else
#define LAImage NSImage
#define LABeginImage(result, size) result = [[LAImage alloc] initWithSize:size]; [result lockFocusWithScale:LAGraphicsGetImageScale()]
#define LAEndImage(result) [result unlockFocusWithScale];
NSData * LAImageJPEGRepresentation(LAImage *image, CGFloat compressionQuality);
NSData * LAImagePNGRepresentation(LAImage *image);
#define LABlendMode NSCompositingOperation
#define LABlendModeClear NSCompositeClear
#define LABlendModeSourceOver NSCompositeSourceOver
#define LABlendModeSourceIn NSCompositeSourceIn
#define LABlendModeSourceOut NSCompositeSourceOut
#define LABlendModeSourceAtop NSCompositeSourceAtop
#define LABlendModeDestinationOver NSCompositeDestinationOver
#define LABlendModeDestinationIn NSCompositeDestinationIn
#define LABlendModeDestinationOut NSCompositeDestinationOut
#define LABlendModeDestinationAtop NSCompositeDestinationAtop
#endif

@interface LAImage (LAImageAdditions)

#if !TARGET_OS_IPHONE
+ (LAImage *)imageWithData:(NSData *)data;
+ (LAImage *)imageWithData:(NSData *)data scale:(CGFloat)scale;
#endif
+ (LAImage *)imageNamed:(NSString *)name inResourcesBundle:(NSBundle *)bundle;
+ (LAImage *)libraryImageNamed:(NSString *)name;
#if !TARGET_OS_IPHONE
- (CGImageRef)CGImage;
- (CGFloat)scale;
- (void)lockFocusWithScale:(CGFloat)scale;
- (void)unlockFocusWithScale;
- (void)drawAtPoint:(LAPoint)point;
- (void)drawAtPoint:(LAPoint)point blendMode:(LABlendMode)blendMode alpha:(CGFloat)alpha;
- (NSImageRep *)bestRepresentationForRect:(LARect)rect;
- (void)drawInRect:(LARect)rect;
- (void)drawInRect:(LARect)rect blendMode:(LABlendMode)blendMode alpha:(CGFloat)alpha;
- (void)stretchInRect:(LARect)rect withCapInsets:(LAEdgeInsets)capInsets alpha:(CGFloat)alpha;
#endif
- (void)drawInRect:(LARect)rect fromRect:(LARect)sourceRect blendMode:(LABlendMode)blendMode alpha:(CGFloat)alpha;
- (LASize)maxProportionalSizeForSize:(LASize)maxSize;
- (void)drawAtPoint:(LAPoint)point withMaxSize:(LASize)size;
- (void)drawInMaxRect:(LARect)rect;
- (LAImage *)resizedImageWithMaxSize:(LASize)size;
- (LAImage *)imageToFillSize:(LASize)sizeToFill;
- (LAImage *)compositedImageWithMask:(LAImage *)mask background:(LAImage *)background overlay:(LAImage *)overlay;
- (LAImage *)imageByAddingIcon:(LAImage *)icon;
- (LAImage *)reflectedImage;
- (LAImage *)roundedImageWithSize:(LASize)size cornerRadius:(CGFloat)cornerRadius;
- (LAImage *)darkenedImageWithLevel:(CGFloat)level;
- (LAImage *)disabledImageWithLevel:(CGFloat)level;
- (LAImage *)transparentImageWithAlpha:(CGFloat)alpha;
- (LAImage *)tintedImageWithColor:(LAColor *)color usingCache:(BOOL)useCache;
+ (LAImage *)gradient:(LAGradient *)gradient withSize:(LASize)size;
+ (LAImage *)progressGradientWithSize:(LASize)size roundedCorners:(LARectCorner)roundedCorners color:(LAColor *)color;
+ (LAImage *)badge:(NSString *)badge withMaxSize:(LASize)size radius:(CGFloat)radius color:(LAColor *)color backgroundColor:(LAColor *)backgroundColor icon:(LAImage *)icon;
+ (LAImage *)gradientBadge:(NSString *)badge withMaxSize:(LASize)size radius:(CGFloat)radius color:(LAColor *)color backgroundColor:(LAColor *)backgroundColor;
+ (LAImage *)textIcon:(NSString *)text withSize:(LASize)size color:(LAColor *)color;
+ (LAImage *)colorIcon:(LAColor *)color withSize:(LASize)size;
+ (LAImage *)roundedRectWithSize:(LASize)size cornerRadius:(CGFloat)cornerRadius color:(LAColor *)color;
#if !TARGET_OS_IPHONE
+ (LAImage *)imageWithPreviewOfFileAtPath:(NSString *)path ofSize:(LASize)size asIcon:(BOOL)icon;
#endif

@end
