//
//  LAObserver.h
//  LAAdditions
//
//  Created by Cyril Anger on 17/03/08.
//  Copyright 2011 LyricApps. All rights reserved.
//

#import <LAAdditions/LAOrderedMap.h>
#import <LAAdditions/LADatabaseObject.h>

@protocol LAObserverDelegate;

@interface LAObserver : NSObject

@property (readonly) BOOL shouldUpdate;
@property (strong, nonatomic) NSArray *observedObjects;
@property (strong) id observedObject;
@property (strong, nonatomic, readonly) NSString *observedObjectsChildrenKeyPath;
@property (strong, nonatomic) NSSet *observedObjectsDelegateKeyPaths;
@property (strong, nonatomic, readonly) NSString *childrenKeyPath;
@property (strong, nonatomic, readonly) NSString *leafKeyPath;
@property (strong, nonatomic) NSArray *children;
@property (strong, nonatomic) NSSet *updateKeyPaths;
@property (strong, nonatomic) NSSet *delegateKeyPaths;
@property (strong, nonatomic) NSPredicate *filterPredicate;
@property (strong, nonatomic) NSArray *sortDescriptors;
@property (nonatomic) BOOL providesIncrementalUpdates;
@property (weak, nonatomic) id <LAObserverDelegate> delegate;
@property (readonly) NSArray *objects;
@property (readonly) NSArray *filteredObjects;
@property (readonly) NSArray *allObjects;

- (id)initWithObservedObjectsChildrenKeyPath:(NSString *)observedObjectsChildrenKeyPath childrenKeyPath:(NSString *)childrenKeyPath leafKeyPath:(NSString *)leafKeyPath;
- (id)initWithChildren:(NSArray *)children;
- (void)enableUpdates;
- (void)disableUpdates;
- (void)makeUpdates;
- (id)associatedObject:(id)object;
- (id)parent:(id)object;
- (BOOL)isLeaf:(id)object;
- (BOOL)isFiltered:(id)object;
- (BOOL)isHidden:(id)object;
- (NSInteger)level:(id)object;

@end

@protocol LAObserverDelegate <NSObject>

- (void)observerWillBeginUpdates:(LAObserver *)observer;
- (NSArray *)observer:(LAObserver *)observer representedObject:(id)representedObject associatedTo:(id)associatedObject didAddChildren:(NSArray *)children atIndexes:(NSIndexSet *)indexes;
- (void)observer:(LAObserver *)observer representedObject:(id)representedObject associatedTo:(id)associatedObject didRemoveChildrenAtIndexes:(NSIndexSet *)indexes;
- (void)observer:(LAObserver *)observer childAtIndex:(NSUInteger)oldIndex ofRepresentedObject:(id)oldRepresentedObject associatedTo:(id)oldAssociatedObject didMoveToIndex:(NSUInteger)newIndex ofRepresentedObject:(id)newRepresentedObject associatedTo:(id)newAssociatedObject;
- (void)observerDidEndUpdates:(LAObserver *)observer;

@optional

- (void)observer:(LAObserver *)observer representedObjectDidChangeState:(id)representedObject associatedTo:(id)associatedObject;
- (void)observer:(LAObserver *)observer observedObject:(id)observedObject didChangeValueForKeyPath:(NSString *)keyPath;
- (void)observer:(LAObserver *)observer representedObject:(id)representedObject associatedTo:(id)associatedObject didChangeValueForKeyPath:(NSString *)keyPath;

@end
