//
//  LAOutlineView.h
//  LAAdditions
//
//  Created by Cyril Anger on 06/03/11.
//  Copyright 2011 LyricApps. All rights reserved.
//

#import <Foundation/Foundation.h>

#import <LAAdditions/LAColumn.h>

@interface LAOutlineView : NSOutlineView <NSMenuDelegate>

@property (nonatomic, readonly) IBInspectable BOOL allowsCustomization;
@property (nonatomic) IBInspectable BOOL makeFirstResponderOnFirstClick;
@property (strong, nonatomic) IBInspectable NSColor *rowBackgroundColor;
@property (strong, nonatomic) IBInspectable NSColor *alternateRowBackgroundColor;
@property (nonatomic, readonly) NSArray *visibleTableColumns;

@property (weak, nonatomic) IBOutlet NSLayoutConstraint *automaticHeightConstraint;

- (void)noteHeightOfRowsChanged;
- (void)scrollRowIndexesToVisible:(NSIndexSet *)indexes;
- (void)restoreExpandedItems;
- (void)saveEditedItem;
- (void)restoreEditedItem;

- (IBAction)expand:(id)sender;
- (IBAction)expandAll:(id)sender;
- (IBAction)collapse:(id)sender;
- (IBAction)collapseAll:(id)sender;

@end

@protocol LAOutlineViewDelegate <NSOutlineViewDelegate>

@optional

- (BOOL)outlineView:(NSOutlineView *)outlineView validateProposedFirstResponder:(NSResponder *)responder inView:(NSView *)view forTableColumn:(NSTableColumn *)tableColumn item:(id)item;
- (void)outlineView:(NSOutlineView *)outlineView deleteItems:(NSArray *)items;
- (NSViewController *)outlineView:(NSOutlineView *)outlineView previewViewControllerForItems:(NSArray *)items;
- (void)outlineView:(NSOutlineView *)outlineView draggingSession:(NSDraggingSession *)session movedToPoint:(NSPoint)screenPoint;

@end

@interface NSArray (LATableColumnAdditions)

- (NSArray *)duplicatedTableColumns;
- (BOOL)sizeTableColumnsToFitWidth:(CGFloat)width withIntercolumnSpacing:(CGFloat)intercolumnSpacing;

@end

@interface LATableColumn : NSTableColumn

@property (getter = isHideable, nonatomic) BOOL hideable;
@property (strong, nonatomic) id representedObject;

- (instancetype)initWithIdentifier:(NSUserInterfaceItemIdentifier)identifier title:(NSString *)title;

@end

@interface LATableHeaderView : NSTableHeaderView

@property (strong, nonatomic) NSArray *columnsSets;
@property (nonatomic, readonly) CGFloat lineHeight;

- (LAColumnsSet *)columnsSetNamed:(NSString *)name;
- (void)drawBackgroundInRect:(LARect)rect;
- (void)drawSeparatorInRect:(LARect)rect;
- (void)drawColumn:(LAColumn *)column inRect:(LARect)rect;
- (void)drawColumnTitle:(NSString *)title withAlignment:(NSTextAlignment)alignment inRect:(LARect)rect;

@end
