//
//  LAPrintView.h
//  LAAdditions
//
//  Created by Cyril Anger on 13/10/07.
//  Copyright 2011 LyricApps. All rights reserved.
//

#import <LAAdditions/LAOutlineView.h>
#import <LAAdditions/LAColumn.h>
#import <LAAdditions/LAStringAdditions.h>

@class LAPrintViewSettingsController;

typedef LASize (^LAPrintViewComputeSizeBlock)(id printView);
typedef void (^LAPrintViewDrawingBlock)(id printView, LARect rect);
typedef NSUInteger (^LATablePrintViewComputeColumnsSetsBlock)(id printView);
typedef void (^LAPrintViewSettingsChangedBlock)(id printView);

@interface LAPrintView : NSView

@property (strong, nonatomic) NSString *header;
@property (weak, nonatomic) NSPrintInfo *printInfo;
@property (strong, nonatomic) LAPrintViewSettingsController *settingsController;

- (id)initWithHeader:(NSString *)header computeSizeBlock:(LAPrintViewComputeSizeBlock)computeSizeBlock drawingBlock:(LAPrintViewDrawingBlock)drawingBlock;
- (void)updateFrame;

@end

@interface LATablePrintView : LAPrintView

@property (nonatomic, readonly) CGFloat pageWidth;
@property (nonatomic, readonly) CGFloat pageHeight;
@property (nonatomic) CGFloat lineHeight;
@property (nonatomic, readonly) NSUInteger numberOfLinesPerPage;
@property (nonatomic) NSUInteger numberOfLines;
@property (nonatomic) NSUInteger numberOfColumnsSets;
@property (nonatomic) NSUInteger numberOfExtraPages;
@property (nonatomic, readonly) NSUInteger numberOfPages;

- (id)initWithHeader:(NSString *)header numberOfLines:(NSUInteger)numberOfLines numberOfExtraPages:(NSUInteger)numberOfExtraPages computeColumnsSetsBlock:(LATablePrintViewComputeColumnsSetsBlock)computeColumnsSetsBlock drawingBlock:(LAPrintViewDrawingBlock)drawingBlock;
- (LARect)rectForLine:(NSUInteger)line;
- (CGFloat)availableWidthForNumberOfColumns:(NSInteger)numberOfColumns;
- (NSArray *)splitTableColumns:(NSArray *)tableColumns repeatingFirstTableColumn:(BOOL)repeatFirstTableColumn;
- (NSArray *)resizedTableColumns:(NSArray *)tableColumns;
- (NSArray *)splitColumnsSet:(LAColumnsSet *)columnsSet repeatingFirstColumn:(BOOL)repeatFirstColumn;
- (LAColumnsSet *)resizedColumnsSet:(LAColumnsSet *)columnsSet;
- (void)drawSeparatorForLine:(NSUInteger)line withColor:(LAColor *)color;
- (void)drawHeaderForTableColumns:(NSArray *)columns inRect:(LARect)rect withFont:(LAFont *)font color:(LAColor *)color;
- (void)drawHeaderForColumnsSet:(LAColumnsSet *)columnsSet inRect:(LARect)rect withFont:(LAFont *)font color:(LAColor *)color;

@end

@interface LAPrintViewSettingsController : NSViewController <NSPrintPanelAccessorizing>

@property (weak, nonatomic) LAPrintView *printView;
@property (strong, nonatomic, readonly) NSDictionary *defaultSettings;

- (id)initWithNibName:(NSString *)nibName bundle:(NSBundle *)nibBundle defaultSettings:(NSDictionary *)defaultSettings settingsChangedBlock:(LAPrintViewSettingsChangedBlock)settingsChangedBlock;

@end
