//
//  LASectionsController.h
//  LAAdditions
//
//  Created by Cyril Anger on 28/02/11.
//  Copyright 2011 LyricApps. All rights reserved.
//

#import <LAAdditions/LAViewController.h>
#import <LAAdditions/LATreeNode.h>
#import <LAAdditions/LATreeNodesDataSource.h>
#import <LAAdditions/LAOutlineView.h>

@class LASectionsController;

@interface LASection : NSObject

@property (weak, nonatomic, readonly) LASectionsController *controller;
@property (strong, nonatomic, readonly) NSString *name;
@property (strong, nonatomic) NSString *title;
@property (strong, nonatomic) NSString *comment;
@property (nonatomic) BOOL hidden;
@property (strong, nonatomic, readonly) NSMutableArray *dataSources;
@property (nonatomic, readonly) NSArray *allNodes;
@property (strong, nonatomic) LATableCellView *view;
@property (strong, nonatomic) NSString *viewIdentifier;
@property (nonatomic, readonly) BOOL shouldUpdateTreeView;

- (id)initWithName:(NSString *)name title:(NSString *)title;
- (LATreeNodesDataSource *)dataSourceNamed:(NSString *)name;
- (void)insertDataSource:(LATreeNodesDataSource *)dataSource atIndex:(NSUInteger)index;
- (void)addDataSource:(LATreeNodesDataSource *)dataSource;
- (void)removeDataSourceNamed:(NSString *)name;
- (NSUInteger)numberOfNodes;
- (LATreeNode *)nodeAtIndex:(NSUInteger)index;
- (void)enableTreeViewUpdates;
- (void)disableTreeViewUpdates;

@end

@interface LASectionsController : LAViewController <NSOutlineViewDataSource, LAOutlineViewDelegate>

@property (nonatomic) BOOL usesClassicDrag;
@property (strong, nonatomic, readonly) NSMutableArray *sections;
@property (strong, nonatomic, readonly) NSArray *visibleSections;
@property (weak, nonatomic) LATreeNodesDataSource *selectedDataSource;

@property (weak, nonatomic) IBOutlet LAOutlineView *treeView;

- (void)loadSections;
- (void)createSections;
- (LASection *)sectionNamed:(NSString *)name;
- (void)insertSection:(LASection *)section atIndex:(NSUInteger)index;
- (LASection *)insertSectionNamed:(NSString *)name atIndex:(NSUInteger)index;
- (void)addSection:(LASection *)section;
- (LASection *)addSectionNamed:(NSString *)name;
- (LASection *)removeSectionNamed:(NSString *)name;
- (void)invalidateVisibleSections;
- (LATreeNodesDataSource *)dataSourceNamed:(NSString *)name;
- (NSIndexSet *)convertIndexes:(NSIndexSet *)indexes fromDataSource:(LATreeNodesDataSource *)dataSource;
- (id)parentForDataSource:(LATreeNodesDataSource *)dataSource;
- (LATreeNode *)nodeWithID:(NSString *)ID;
- (void)registerDraggedTypes;
- (LATableCellView *)cellForComputingRowHeightWithIdentifier:(NSString *)identifier;

@end

@interface LASingleDataSourceSectionsController : LASectionsController

@property (strong, nonatomic) IBOutlet LATreeNodesDataSource *dataSource;

@end
