//
//  LASynchronizedObject.h
//  LAAdditions
//
//  Created by Cyril Anger on 08/11/12.
//  Copyright (c) 2012 LyricApps. All rights reserved.
//

#import <Foundation/Foundation.h>

#import <LAAdditions/LAObject.h>
#import <LAAdditions/LARequestsMap.h>
#import <LAAdditions/LAImageAdditions.h>

extern NSString * LALocalizedSynchronizationString(NSString *key);

@protocol LASynchronizedObject <LAObject>

@property (nonatomic, readonly) NSString *synchronizationDescription;
@property (nonatomic, readonly) BOOL shouldCreateIfMissing;
@property (nonatomic, readonly) NSString *synchronizationParentID;
@property (nonatomic, readonly) BOOL shouldSynchronizeProperties;
@property (strong, nonatomic, readonly) NSDate *lastModificationDate;
@property (weak, nonatomic, readonly) id originalObject;

@end

@interface NSArray (LASynchronizedObjectAdditions)

- (NSArray *)reorderedArrayWithIDs:(NSArray *)IDs;
- (NSUInteger)indexToInsertObject:(id <LAObject>)object conservingOrder:(NSArray *)IDs;

@end

typedef BOOL (^LASynchronizationChangeCompareBlock)(id value, id otherValue);
typedef NSString * (^LASynchronizationChangeDescriptionBlock)(id object, id value);
typedef LAImage * (^LASynchronizationChangeImageBlock)(id object, id value);

@interface LASynchronizationChange : NSObject <LAObject>

@property (strong, nonatomic) NSString *title;
@property (strong, nonatomic) NSString *value;
@property (strong, nonatomic) LAImage *image;
@property (copy, nonatomic) void (^block)(void);
@property (strong, nonatomic) NSString *otherValue;
@property (strong, nonatomic) LAImage *otherImage;
@property (copy, nonatomic) void (^otherBlock)(void);
@property (nonatomic) BOOL keepOther;

- (id)initWithTitle:(NSString *)title;
+ (instancetype)compareValue:(id)value ofObject:(id <LASynchronizedObject>)object withValue:(id)otherValue ofObject:(id <LASynchronizedObject>)otherObject localizationKey:(NSString *)key block:(void (^)(id object, id value))block compareBlock:(LASynchronizationChangeCompareBlock)compareBlock descriptionBlock:(LASynchronizationChangeDescriptionBlock)descriptionBlock imageBlock:(LASynchronizationChangeImageBlock)imageBlock;
+ (instancetype)compareValue:(id)value ofObject:(id <LASynchronizedObject>)object withValue:(id)otherValue ofObject:(id <LASynchronizedObject>)otherObject localizationKey:(NSString *)key block:(void (^)(id object, id value))block descriptionBlock:(LASynchronizationChangeDescriptionBlock)descriptionBlock imageBlock:(LASynchronizationChangeImageBlock)imageBlock;
+ (instancetype)compareValue:(id)value ofObject:(id <LASynchronizedObject>)object withValue:(id)otherValue ofObject:(id <LASynchronizedObject>)otherObject localizationKey:(NSString *)key block:(void (^)(id object, id value))block descriptionBlock:(LASynchronizationChangeDescriptionBlock)descriptionBlock;
+ (instancetype)compareValue:(id)value ofObject:(id <LASynchronizedObject>)object withValue:(id)otherValue ofObject:(id <LASynchronizedObject>)otherObject localizationKey:(NSString *)key block:(void (^)(id object, id value))block;
+ (instancetype)compareBool:(BOOL)value ofObject:(id <LASynchronizedObject>)object withBool:(BOOL)otherValue ofObject:(id <LASynchronizedObject>)otherObject localizationKey:(NSString *)key block:(void (^)(id object, BOOL value))block;
+ (instancetype)compareInt:(int)value ofObject:(id <LASynchronizedObject>)object withInt:(int)otherValue ofObject:(id <LASynchronizedObject>)otherObject localizationKey:(NSString *)key block:(void (^)(id object, int value))block;
+ (instancetype)compareInt64:(long long)value ofObject:(id <LASynchronizedObject>)object withInt64:(long long)otherValue ofObject:(id <LASynchronizedObject>)otherObject localizationKey:(NSString *)key block:(void (^)(id object, long long value))block;
+ (instancetype)compareDouble:(int)value ofObject:(id <LASynchronizedObject>)object withDouble:(int)otherValue ofObject:(id <LASynchronizedObject>)otherObject localizationKey:(NSString *)key block:(void (^)(id object, double value))block;
+ (instancetype)compareDate:(NSDate *)value ofObject:(id <LASynchronizedObject>)object withDate:(NSDate *)otherValue ofObject:(id <LASynchronizedObject>)otherObject localizationKey:(NSString *)key block:(void (^)(id object, NSDate *date))block;
+ (instancetype)compareAmount:(NSDecimalNumber *)value ofObject:(id <LASynchronizedObject>)object withAmount:(NSDecimalNumber *)otherValue ofObject:(id <LASynchronizedObject>)otherObject localizationKey:(NSString *)key block:(void (^)(id object, NSDecimalNumber *amount))block currencyKeyPath:(NSString *)currencyKeyPath;
+ (instancetype)comparePercentage:(NSDecimalNumber *)value ofObject:(id <LASynchronizedObject>)object withPercentage:(NSDecimalNumber *)otherValue ofObject:(id <LASynchronizedObject>)otherObject localizationKey:(NSString *)key block:(void (^)(id object, NSDecimalNumber *value))block;
+ (instancetype)compareImage:(LAImage *)value ofObject:(id <LASynchronizedObject>)object withImage:(LAImage *)otherValue ofObject:(id <LASynchronizedObject>)otherObject localizationKey:(NSString *)key block:(void (^)(id object, LAImage *image))block maxSize:(LASize)maxSize;
+ (instancetype)compareColor:(LAColor *)value ofObject:(id <LASynchronizedObject>)object withColor:(LAColor *)otherValue ofObject:(id <LASynchronizedObject>)otherObject localizationKey:(NSString *)key block:(void (^)(id object, LAColor *color))block;
+ (instancetype)compareCollection:(id)value ofObject:(id <LASynchronizedObject>)object withCollection:(id)otherValue ofObject:(id <LASynchronizedObject>)otherObject localizationKey:(NSString *)key block:(void (^)(id object, id collection))block;
+ (instancetype)compareOrderBetweenArray:(NSArray *)array ofObject:(id <LASynchronizedObject>)object andArray:(NSArray *)otherArray ofObject:(id <LASynchronizedObject>)otherObject localizationKey:(NSString *)key block:(void (^)(id object, NSArray *IDs))block;
- (void)applyChoice;

@end

@interface LASynchronizationChangesGroup : NSObject <LAObject>

@property (strong, nonatomic, readonly) NSString *title;
@property (strong, nonatomic, readonly) NSArray *changes;

- (id)initWithTitle:(NSString *)title changes:(NSArray *)changes;

@end
