//
//  LATCPConnection.h
//  LAAdditions
//
//  Created by Cyril Anger on 17/04/08.
//  Copyright 2008 Angeman. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <CFNetwork/CFNetwork.h>
#import <sys/socket.h>

#define LAHTML(file) [NSString stringWithContentsOfFile:[[NSBundle mainBundle] pathForResource:[file stringByDeletingPathExtension] ofType:[file pathExtension]] encoding:NSUTF8StringEncoding error:nil]
#define LAHTMLWithFormat(file, ...) [NSString stringWithFormat:LAHTML(file), ##__VA_ARGS__]

@class LATCPServer;

@interface LATCPConnection : NSObject

@property (getter = isValid, readonly) BOOL valid;
@property (readonly) NSString *localAddress;
@property (readonly) NSUInteger localPort;
@property (readonly) NSString *remoteAddress;
@property (readonly) NSUInteger remotePort;
@property (weak, nonatomic, readonly) LATCPServer *server;
@property (strong, nonatomic, readonly) NSString *authorizationID;
@property (strong, nonatomic) id context;
@property (weak, nonatomic) id delegate;

- (id)initWithReadStream:(CFReadStreamRef)readStream writeStream:(CFWriteStreamRef)writeStream server:(LATCPServer *)server delegate:(id)delegate;
- (id)initWithSocketHandle:(int)socket server:(LATCPServer *)server delegate:(id)delegate;
- (id)initWithRemoteHostName:(NSString *)name port:(UInt16)port delegate:(id)delegate;
- (id)initWithRemoteAddress:(const struct sockaddr *)address delegate:(id)delegate;
- (void)invalidate;
- (void)detachFromServer;
- (void)receiveData;
- (BOOL)readIntoData:(NSMutableData *)data;
- (BOOL)readIntoFile:(NSString *)file length:(NSUInteger)length;
- (BOOL)writeData:(NSData *)data;
- (BOOL)writeFile:(NSString *)file;

@end

@protocol LATCPConnectionDelegate <NSObject>

@optional

- (void)connectionDidOpen:(LATCPConnection *)connection;
- (void)connectionDidClose:(LATCPConnection *)connection;
- (void)connection:(LATCPConnection *)connection didReachProgress:(float)progress;

@end

@interface LAHTTPConnection : LATCPConnection

@property (strong, nonatomic) NSMutableDictionary *cookies;

- (void)sendHTML:(NSString *)source withStatusCode:(NSInteger)statusCode completionHandler:(void (^)(BOOL success))completionHandler;
- (void)sendFile:(NSString *)file withName:(NSString *)name completionHandler:(void (^)(BOOL success))completionHandler;
- (void)sendRedirectToLocation:(NSString *)location completionHandler:(void (^)(BOOL success))completionHandler;

@end

@protocol LAHTTPConnectionDelegate <LATCPConnectionDelegate>

@optional

- (void)connection:(LAHTTPConnection *)connection didReceiveRequestForURL:(NSURL *)URL withParameters:(NSDictionary *)parameters files:(NSDictionary *)files;

@end

@interface LADataConnection : LATCPConnection

@property (nonatomic, readonly) BOOL wasTerminated;
@property (nonatomic) BOOL usesCompression;
@property (strong, nonatomic, readonly) NSString *application;
@property (strong, nonatomic, readonly) NSString *version;

- (void)terminate;
- (void)sendMessage:(NSDictionary *)message files:(NSDictionary *)files completionHandler:(void (^)(BOOL success))completionHandler;

@end

@protocol LADataConnectionDelegate <LATCPConnectionDelegate>

@optional

- (void)connection:(LADataConnection *)connection didReceiveMessage:(NSDictionary *)message files:(NSDictionary *)files;

@end
