//
//  LATreeNodesDataSource.h
//  LAAdditions
//
//  Created by Cyril Anger on 27/03/11.
//  Copyright 2011 LyricApps. All rights reserved.
//

#import <LAAdditions/LATreeNode.h>
#import <LAAdditions/LAObserver.h>
#import <LAAdditions/LATableRowView.h>
#import <LAAdditions/LAOutlineView.h>

@class LATreeNodesDataSource;
@class LASection;
@class LASectionsController;
@class LAFiltersBar;

typedef void (^LATreeNodesDataSourceBlock)(LATreeNodesDataSource *dataSource, LATreeNode *item, id object);

@interface LATreeNodesDataSource : NSObject <LAObserverDelegate, LAColumnsSetDelegate>

@property (weak, nonatomic) LASection *section;
@property (weak, nonatomic) LASectionsController *controller;
@property (strong, nonatomic, readonly) NSString *name;
@property (nonatomic) BOOL hidden;
@property (nonatomic, readonly) NSArray *registeredNodes;
@property (strong, nonatomic, readonly) NSMutableArray *nodes;
@property (strong, nonatomic, readonly) NSArray *rows;
@property (nonatomic, readonly) BOOL updatesSiblings;
@property (strong, nonatomic) LAObserver *observer;
@property (strong, nonatomic) LADatabaseManager *databaseManager;
@property (weak, nonatomic) Class objectsClass;
@property (weak, nonatomic) Class groupsClass;
@property (nonatomic) BOOL canSelect;
@property (nonatomic) BOOL canSelectMultipleNodes;
@property (strong, nonatomic) NSArray *selectedNodes;
@property (strong, nonatomic) NSArray *selectedObjects;
@property (nonatomic, readonly) NSArray *visibleCells;
@property (nonatomic) BOOL canEdit;
+ (NSSet *)keyPathsForValuesAffectingCanAdd;
@property (nonatomic, readonly) BOOL canAdd;
+ (NSSet *)keyPathsForValuesAffectingCanAddGroup;
@property (nonatomic, readonly) BOOL canAddGroup;
+ (NSSet *)keyPathsForValuesAffectingCanRemove;
@property (nonatomic, readonly) BOOL canRemove;
@property (nonatomic) BOOL deleteObjectsOnRemove;
@property (nonatomic) BOOL showsRemoveConfirmation;
@property (nonatomic) BOOL canBeOrdered;
@property (nonatomic, readonly) BOOL canOrder;
@property (nonatomic, readonly) BOOL canExportToCSV;
@property (nonatomic, readonly) NSArray *tableColumnsForExporting;
@property (nonatomic, readonly) NSString *fileNameForExporting;
@property (nonatomic, readonly) NSString *objectsPasteboardType;
@property (nonatomic, readonly) NSArray *copyPasteboardTypes;
@property (nonatomic, readonly) NSArray *pastePasteboardTypes;
@property (nonatomic, readonly) NSArray *dragPasteboardTypes;
@property (weak, nonatomic) LATreeNodesDataSource *detailsDataSource;
@property (copy, nonatomic) LATreeNodesDataSourceBlock doubleClickHandler;
@property (nonatomic) BOOL callDoubleClickHandlerOnAddNewObject;
@property (nonatomic, readonly) BOOL shouldUpdateTreeView;
@property (nonatomic, readonly) NSArray *sortKeyPaths;
@property (copy, nonatomic) NSString *searchString;
@property (nonatomic, readonly) BOOL repeatsFirstTableColumnWhenPrinting;

@property (weak, nonatomic) IBOutlet LAFiltersBar *filtersBar;

+ (Class)nodeClass;
+ (instancetype)dataSourceWithNode:(LATreeNode *)node;

- (instancetype)initWithName:(NSString *)name;
- (LATreeNode *)createNodeWithRepresentedObject:(id <LAObject>)object;
- (LATreeNode *)nodeWithID:(NSString *)ID;
- (void)registerNodes:(NSArray *)nodes;
- (void)unregisterNodes:(NSArray *)nodes;
- (void)insertNodes:(NSArray *)nodes atIndexes:(NSIndexSet *)indexes;
- (void)removeNodesAtIndexes:(NSIndexSet *)indexes;
- (void)computeRows;
- (BOOL)invalidateRows;
- (void)observedObjectsDidChange;
- (void)selectAllNodes;
- (void)setSelectedObjects:(NSArray *)selectedObjects updateScrollLocation:(BOOL)updateScrollLocation;
- (id)objectForItem:(LATreeNode *)item;
- (CGFloat)heightOfRowByItem:(LATreeNode *)item;
- (LATableCellView *)viewForItem:(LATreeNode *)item withIdentifier:(NSString *)identifier;
- (LATableRowView *)rowViewForItem:(LATreeNode *)item;
- (BOOL)shouldShowOutlineCellForItem:(LATreeNode *)item;
- (BOOL)validateProposedFirstResponder:(NSResponder *)responder inView:(LATableCellView *)view forItem:(LATreeNode *)item withIdentifier:(NSString *)identifier;
- (void)reloadData;
- (id)newObject;
- (id)newGroup;
- (void)insertObjects:(NSArray *)objects inParent:(id)parent atIndexes:(NSIndexSet *)indexes;
- (void)addObjects:(NSArray *)objects;
- (void)addNewObject:(id <LAObject>)newObject;
- (void)removeObjects:(NSArray *)objects forMoving:(BOOL)moving;
- (NSViewController *)previewViewControllerForObjects:(NSArray *)objects;
- (NSString *)keyPathForColumn:(LAColumn *)column;
- (NSString *)keyPathForTableColumn:(LATableColumn *)tableColumn;
- (id)valueForObject:(id)object tableColumn:(LATableColumn *)tableColumn;
- (NSString *)CSVDataForObjects:(NSArray *)objects withSeparator:(NSString *)separator;
- (void)copyObjectsToPasteboard:(NSArray *)objects;
- (void)pasteObjectsFromPasteboard;
- (BOOL)writeItems:(NSArray *)items toPasteboard:(NSPasteboard *)pasteboard;
- (NSPasteboardItem *)pasteboardItemForItem:(LATreeNode *)item;
- (NSArray *)draggedItems:(id <NSDraggingInfo>)info;
- (NSDragOperation)validateDrop:(id <NSDraggingInfo>)info forItem:(LATreeNode *)item childIndex:(NSInteger)index;
- (BOOL)acceptDrop:(id <NSDraggingInfo>)info forItem:(LATreeNode *)item childIndex:(NSInteger)index;
- (NSArray *)namesOfPromisedFilesDroppedAtDestination:(NSURL *)dropDestination forDraggedItems:(NSArray *)items;
- (void)enableTreeViewUpdates;
- (void)disableTreeViewUpdates;
- (NSString *)sortKeyPathForColumn:(LAColumn *)column;
- (void)updateSortDescriptors:(NSArray *)sortDescriptors;
- (void)saveSortDescriptors;
- (void)restoreSortDescriptors;
- (void)sortDescriptorsDidChange;
- (void)updateFilterPredicate;
- (LARect)drawObject:(id)object inRect:(LARect)rect forTableColumn:(LATableColumn *)tableColumn;

- (IBAction)add:(id)sender;
- (IBAction)addGroup:(id)sender;
- (IBAction)remove:(id)sender;
- (IBAction)editSortDescriptors:(id)sender;
- (IBAction)print:(id)sender;

@end

@interface LAStandardTreeNodesDataSource : LATreeNodesDataSource

@end
