//
//  LAViewAdditions.h
//  LAAdditions
//
//  Created by Cyril Anger on 06/06/08.
//  Copyright 2011 LyricApps. All rights reserved.
//

#import <LAAdditions/LAImageAdditions.h>

#if TARGET_OS_IPHONE
#define LAView UIView
#define LAControl UIControl
#define LAViewAutoresizing UIViewAutoresizing
#define LAViewAutoresizingNone UIViewAutoresizingNone
#define LAViewAutoresizingFlexibleLeftMargin UIViewAutoresizingFlexibleLeftMargin
#define LAViewAutoresizingFlexibleWidth UIViewAutoresizingFlexibleWidth
#define LAViewAutoresizingFlexibleRightMargin UIViewAutoresizingFlexibleRightMargin
#define LAViewAutoresizingFlexibleTopMargin UIViewAutoresizingFlexibleTopMargin
#define LAViewAutoresizingFlexibleHeight UIViewAutoresizingFlexibleHeight
#define LAViewAutoresizingFlexibleBottomMargin UIViewAutoresizingFlexibleBottomMargin
#define LALayoutPriority UILayoutPriority
#define LALayoutPriorityRequired UILayoutPriorityRequired
#define LALayoutPriorityDefaultHigh UILayoutPriorityDefaultHigh
#define LALayoutPriorityDefaultLow UILayoutPriorityDefaultLow
#define LAViewNoIntrinsicMetric UIViewNoIntrinsicMetric
#else
#define LAView NSView
#define LAControl NSControl
#define LAViewAutoresizing NSUInteger
#define LAViewAutoresizingNone NSViewNotSizable
#define LAViewAutoresizingFlexibleLeftMargin NSViewMinXMargin
#define LAViewAutoresizingFlexibleWidth NSViewWidthSizable
#define LAViewAutoresizingFlexibleRightMargin NSViewMaxXMargin
#define LAViewAutoresizingFlexibleTopMargin NSViewMinYMargin
#define LAViewAutoresizingFlexibleHeight NSViewHeightSizable
#define LAViewAutoresizingFlexibleBottomMargin NSViewMaxYMargin
#define LALayoutPriority NSLayoutPriority
#define LALayoutPriorityRequired NSLayoutPriorityRequired
#define LALayoutPriorityDefaultHigh NSLayoutPriorityDefaultHigh
#define LALayoutPriorityDefaultLow NSLayoutPriorityDefaultLow
#define LAViewNoIntrinsicMetric NSViewNoInstrinsicMetric
#endif

typedef void (^LAAnimationsBlock)(void);

typedef enum {
	LADirectionNone,
	LADirectionLeftToRight,
	LADirectionRightToLeft,
	LADirectionTopToBottom,
	LADirectionBottomToTop
} LADirection;

@interface LAView (LAViewAdditions)

#if TARGET_OS_IPHONE
- (void)adjustCenter:(LAPoint)center;
@property (getter = isHiddenOrHasHiddenAncestor, readonly) BOOL hiddenOrHasHiddenAncestor;
#else
- (void)bringSubviewToFront:(LAView *)view;
- (void)sendSubviewToBack:(LAView *)view;
- (void)setNeedsDisplay;
- (void)setNeedsLayout;
- (void)setNeedsUpdateConstraints;
#endif
@property (getter = hasDarkAppearance, readonly) BOOL darkAppearance;
- (void)updateSubviewsWithFontSizeBlock:(CGFloat (^)(LAView *view))fontSizeBlock;
- (void)resetAnchorPoint;
- (void)shakeWithDelegate:(id)delegate;
- (void)startWobbling;
- (void)stopWobbling;
- (void)makeTransitionToView:(LAView *)newView withDirection:(LADirection)direction completionHandler:(void (^)(void))completionHandler;
- (LAImage *)imageWithSize:(LASize)size;

#if !TARGET_OS_IPHONE
- (IBAction)openMenu:(id)sender;
#endif

@end

#if TARGET_OS_IPHONE

@interface UIControl (LAControlAdditions)

@property (strong, nonatomic) IBInspectable NSString *fieldName;
@property (weak, nonatomic) IBOutlet UILabel *fieldLabel;

@end

#else

@interface NSControl (LAControlAdditions)

@property (strong, nonatomic) IBInspectable NSString *fieldName;
@property (weak, nonatomic) IBOutlet NSTextField *fieldLabel;

@end

@interface NSButton (LAViewAdditions)

- (IBAction)openMenu:(id)sender;

@end

#endif
