//
//  LAWebController.h
//  LAAdditions
//
//  Created by Cyril Anger on 12/06/08.
//  Copyright 2011 LyricApps. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <WebKit/WebKit.h>

#import <MMTabBarView/MMTabBarItem.h>
#import <MMTabBarView/MMTabBarView.h>

#import <LAAdditions/LAViewController.h>
#import <LAAdditions/LATextField.h>
#import <LAAdditions/LABackgroundView.h>

@class LAWebView;

typedef void (^LAWebViewCompletionHandler)(NSError *error);

@interface LAWebView : WebView <WebFrameLoadDelegate, MMTabBarItem>

@property (strong, nonatomic) NSURL *homeURL;
@property (nonatomic, readonly) NSImage *schemeIcon;

- (void)loadRequest:(NSURLRequest *)request withCompletionHandler:(LAWebViewCompletionHandler)completionHandler;
- (void)loadHTMLString:(NSString *)string baseURL:(NSURL *)URL withCompletionHandler:(LAWebViewCompletionHandler)completionHandler;
- (void)loadContent:(NSString *)content withTemplate:(NSString *)template completionHandler:(LAWebViewCompletionHandler)completionHandler;
- (void)loadFile:(NSString *)file withTemplate:(NSString *)template completionHandler:(LAWebViewCompletionHandler)completionHandler;
- (void)goHome;
- (void)sizeToFit;

@end

@protocol LAWebViewDelegate <NSObject>

@optional

- (void)webViewWillShow:(WebView *)webView;
- (void)webViewDidDownloadFile:(NSString *)file;

@end

@interface LAWebController : LAViewController <WebDownloadDelegate, WebFrameLoadDelegate, WebPolicyDelegate, WebResourceLoadDelegate, WebUIDelegate>

@property (strong, nonatomic) NSString *userAgent;
@property (weak, nonatomic, readonly) LAWebView *currentWebView;
@property (strong, nonatomic) NSView *accessoryBarView;
@property (weak, nonatomic) id <LAWebViewDelegate> delegate;

@property (weak, nonatomic) IBOutlet NSImageView *schemeImageView;
@property (weak, nonatomic) IBOutlet LATextField *URLField;
@property (weak, nonatomic) IBOutlet NSSegmentedControl *homeButton;
@property (weak, nonatomic) IBOutlet NSSegmentedControl *navigateButton;
@property (weak, nonatomic) IBOutlet NSButton *reloadButton;
@property (weak, nonatomic) IBOutlet NSSegmentedControl *zoomButton;
@property (weak, nonatomic) IBOutlet MMTabBarView *tabBarView;

- (id)initWithDelegate:(id <LAWebViewDelegate>)delegate;
- (void)selectWebViewWithHomeURL:(NSURL *)homeURL;

- (IBAction)loadURL:(id)sender;
- (IBAction)reload:(id)sender;
- (IBAction)goHome:(id)sender;
- (IBAction)navigate:(id)sender;

@end

@interface LAWebControllerBarView : LABackgroundView

@end
