-- Library

CREATE TABLE ICLibrary (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT
);

-- Currencies

CREATE TABLE ICCurrency (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"index" INTEGER NOT NULL,
	"code" TEXT NOT NULL,
	"symbol" TEXT NOT NULL,
	"fractionDigits" INTEGER NOT NULL,
	"changeRate" TEXT NOT NULL
);

-- Categories

CREATE TABLE ICCategory (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"parent" TEXT NOT NULL,
	"index" INTEGER NOT NULL,
	"name" TEXT NOT NULL,
	"icon" BLOB, "iconScale" INTEGER,
	"color" TEXT NOT NULL,
	"expense" INTEGER NOT NULL,
	"income" INTEGER NOT NULL
);

-- Taxes

CREATE TABLE ICTax (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"index" INTEGER NOT NULL,
	"name" TEXT NOT NULL,
	"rate" TEXT NOT NULL
);

-- Companies

CREATE TABLE ICCompany (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"index" INTEGER NOT NULL,
	"name" TEXT NOT NULL,
	"logo" BLOB, "logoScale" INTEGER,
	"identificationNumber" TEXT,
	"taxesNumber" TEXT,
	"address" TEXT,
	"phone" TEXT,
	"mail" TEXT,
	"webSite" TEXT,
	"nextInvoiceNumber" TEXT,
	"currency" TEXT NOT NULL
);

-- Clients

CREATE TABLE ICClient (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"class" TEXT NOT NULL,
	"name" TEXT NOT NULL,
	"icon" BLOB, "iconScale" INTEGER,
	"hidden" INTEGER NOT NULL,
	"parent" TEXT NOT NULL,
	"currency" TEXT NOT NULL,
	"address" TEXT,
	"phone" TEXT,
	"mail" TEXT
);

-- Invoices

CREATE TABLE ICInvoice (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"company" TEXT NOT NULL,
	"type" TEXT NOT NULL,
	"number" TEXT NOT NULL,
	"date" TEXT NOT NULL,
	"dueDate" TEXT,
	"title" TEXT NOT NULL,
	"comment" TEXT,
	"client" TEXT,
	"currency" TEXT NOT NULL,
	"discount" TEXT
);

CREATE TABLE ICInvoiceItem (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"invoice" TEXT NOT NULL,
	"index" INTEGER NOT NULL,
	"title" TEXT NOT NULL,
	"details" TEXT,
	"amount" TEXT,
	"amountIncludesTaxes" INTEGER NOT NULL,
	"quantity" TEXT,
	"taxesName" TEXT,
	"taxesRate" TEXT,
	"taxesCategory" TEXT,
	"free" INTEGER NOT NULL,
	"category" TEXT
);

CREATE INDEX ICInvoiceItemInvoiceIndex ON ICInvoiceItem ("invoice");

CREATE TABLE ICInvoiceTemplate (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"index" INTEGER NOT NULL,
	"name" TEXT NOT NULL,
	"content" TEXT
);

-- Securities

CREATE TABLE ICSecurity (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"name" TEXT NOT NULL,
	"symbol" TEXT,
	"currency" TEXT NOT NULL,
	"hidden" INTEGER NOT NULL
);

CREATE TABLE ICQuote (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"security" TEXT NOT NULL,
	"date" TEXT NOT NULL,
	"value" TEXT NOT NULL
);

CREATE INDEX ICQuoteSecurityIndex ON ICQuote ("security");

-- Reports

CREATE TABLE ICReport (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"class" TEXT NOT NULL,
	"name" TEXT NOT NULL,
	"icon" BLOB, "iconScale" INTEGER,
	"hidden" INTEGER NOT NULL,
	"parent" TEXT,
	"index" INTEGER NOT NULL,
	"currency" TEXT,
	"transactionsCondition" TEXT,
	"splitsCondition" TEXT,
	"usesValueDate" INTEGER,
	"type" TEXT,
	"groupingKeyPaths" TEXT,
	"startDate" TEXT,
	"period" INTEGER, "periodUnit" INTEGER,
	"addTotalPeriod" INTEGER,
	"addAveragePeriod" INTEGER,
	"groupsByParent" INTEGER,
	"comparisonKeyPath" TEXT
);

-- Budgets

CREATE TABLE ICBudget (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"class" TEXT NOT NULL,
	"name" TEXT NOT NULL,
	"icon" BLOB, "iconScale" INTEGER,
	"hidden" INTEGER NOT NULL,
	"parent" TEXT NOT NULL,
	"index" INTEGER NOT NULL,
	"comment" TEXT,
	"currency" TEXT NOT NULL,
	"accounts" TEXT,
	"remainingTransactionsAccounts" TEXT
);

CREATE TABLE ICBudgetItem (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"budget" TEXT NOT NULL,
	"index" INTEGER NOT NULL,
	"category" TEXT,
	"amount" TEXT NOT NULL,
	"income" INTEGER NOT NULL,
	"startDate" TEXT NOT NULL,
	"endDate" TEXT,
	"duration" INTEGER NOT NULL, "durationUnit" INTEGER NOT NULL
);

CREATE INDEX ICBudgetItemBudgetIndex ON ICBudgetItem ("budget");

CREATE TABLE ICBudgetItemPeriod (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"item" TEXT NOT NULL,
	"index" INTEGER NOT NULL,
	"amount" TEXT NOT NULL,
	"startDate" TEXT NOT NULL,
	"endDate" TEXT NOT NULL
);

CREATE INDEX ICBudgetItemPeriodItemIndex ON ICBudgetItemPeriod ("item");

CREATE TABLE ICBudgetItemPeriodAllocation (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"sender" TEXT NOT NULL,
	"receiver" TEXT NOT NULL,
	"amount" TEXT NOT NULL
);

-- Accounts

CREATE TABLE ICAccount (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"class" TEXT NOT NULL,
	"name" TEXT NOT NULL,
	"icon" BLOB, "iconScale" INTEGER,
	"hidden" INTEGER NOT NULL,
	"parent" TEXT NOT NULL,
	"index" INTEGER NOT NULL,
	"comment" TEXT,
	"currency" TEXT NOT NULL,
	"warningBalance" TEXT NOT NULL,
	"number" TEXT,
	"webSite" TEXT,
	"includedInTotal" INTEGER,
	"type" TEXT,
	"checkingAccountInfo" TEXT,
	"creditCardAccountInfo" TEXT,
	"savingsAccountInfo" TEXT,
	"loanAccountInfo" TEXT,
	"owner" TEXT
);

CREATE TABLE ICOwner (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"parent" TEXT,
	"index" INTEGER NOT NULL,
	"person" TEXT NOT NULL,
	"ratio" TEXT NOT NULL
);

CREATE INDEX ICOwnerParentIndex ON ICOwner ("parent");

-- Checking account infos

CREATE TABLE ICCheckingAccountInfo (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"account" TEXT NOT NULL
);

CREATE TABLE ICCheckbook (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"checkingAccountInfo" TEXT NOT NULL,
	"name" TEXT NOT NULL,
	"nextCheckNumber" TEXT
);

CREATE TABLE ICCard (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"checkingAccountInfo" TEXT NOT NULL,
	"name" TEXT NOT NULL,
	"number" TEXT
);

-- Credit card account infos

CREATE TABLE ICCreditCardAccountInfo (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"account" TEXT NOT NULL,
	"number" TEXT,
	"transfer" TEXT,
	"linkedTransaction" TEXT
);

-- Savings account infos

CREATE TABLE ICSavingsAccountInfo (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"account" TEXT NOT NULL,
	"interest" TEXT
);

CREATE TABLE ICInterestRate (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"savingsAccountInfo" TEXT NOT NULL,
	"date" TEXT NOT NULL,
	"rate" TEXT NOT NULL
);

-- Loan account infos

CREATE TABLE ICLoanAccountInfo (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"account" TEXT NOT NULL,
	"rate" TEXT NOT NULL,
	"transfer" TEXT,
	"linkedTransaction" TEXT,
	"linkedTransactionAmount" TEXT
);

-- Bank statements

CREATE TABLE ICBankStatement (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"account" TEXT NOT NULL,
	"date" TEXT NOT NULL,
	"name" TEXT,
	"startingBalance" TEXT NOT NULL,
	"endingBalance" TEXT NOT NULL
);

CREATE INDEX ICBankStatementAccountIndex ON ICBankStatement ("account");

-- Transactions

CREATE TABLE ICTransaction (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"account" TEXT NOT NULL,
	"date" TEXT NOT NULL,
	"valueDate" TEXT,
	"index" INTEGER NOT NULL,
	"name" TEXT NOT NULL,
	"comment" TEXT,
	"useSumOfSplits" INTEGER NOT NULL,
	"amount" TEXT,
	"amountWithoutTaxes" TEXT,
	"taxesRate" TEXT,
	"payee" TEXT,
	"type" TEXT,
	"number" TEXT,
	"links" BLOB,
	"highlightColor" TEXT,
	"latitude" REAL,
	"longitude" REAL,
	"investmentTransactionInfo" TEXT,
	"scheduledTransaction" TEXT,
	"occurrence" INTEGER,
	"status" TEXT NOT NULL,
	"budgetItemPeriod" TEXT,
	"statement" TEXT,
	"externalID" TEXT
);

CREATE INDEX ICTransactionAccountIndex ON ICTransaction ("account");
CREATE INDEX ICTransactionScheduledTransactionIndex ON ICTransaction ("scheduledTransaction");
CREATE INDEX ICTransactionBudgetItemPeriodIndex ON ICTransaction ("budgetItemPeriod");

CREATE TABLE ICInvestmentTransactionInfo (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"type" TEXT,
	"security" TEXT,
	"shares" TEXT,
	"commission" TEXT
);

CREATE TABLE ICTransactionSplit (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"transaction" TEXT NOT NULL,
	"index" INTEGER NOT NULL,
	"amount" TEXT,
	"amountWithoutTaxes" TEXT,
	"taxesRate" TEXT,
	"taxesCategory" TEXT,
	"ratio" TEXT,
	"comment" TEXT,
	"project" TEXT,
	"category" TEXT,
	"linkedSplit" TEXT,
	"ignoredInBudgets" INTEGER NOT NULL,
	"invoice" TEXT,
	"ignoredInReports" INTEGER NOT NULL,
	"ignoredInAverageBalance" INTEGER NOT NULL,
	"refund" INTEGER NOT NULL,
	"usesAccountOwners" INTEGER NOT NULL,
	"scheduledSplit" TEXT,
	"budgetItemPeriod" TEXT
);

CREATE INDEX ICTransactionSplitTransactionIndex ON ICTransactionSplit ("transaction");
CREATE INDEX ICTransactionSplitScheduledSplitIndex ON ICTransactionSplit ("scheduledSplit");

-- Scheduled transactions

CREATE TABLE ICScheduledTransaction (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"account" TEXT NOT NULL,
	"date" TEXT NOT NULL,
	"valueDate" TEXT,
	"index" INTEGER NOT NULL,
	"name" TEXT NOT NULL,
	"comment" TEXT,
	"useSumOfSplits" INTEGER NOT NULL,
	"amount" TEXT,
	"amountWithoutTaxes" TEXT,
	"taxesRate" TEXT,
	"payee" TEXT,
	"type" TEXT,
	"number" TEXT,
	"links" BLOB,
	"highlightColor" TEXT,
	"latitude" REAL,
	"longitude" REAL,
	"investmentTransactionInfo" TEXT,
	"accountInfo" TEXT,
	"frequency" INTEGER, "frequencyUnit" INTEGER,
	"numberOfOccurrences" INTEGER,
	"nextOccurrence" INTEGER NOT NULL
);

CREATE TABLE ICScheduledTransactionSplit (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"transaction" TEXT NOT NULL,
	"index" INTEGER NOT NULL,
	"amount" TEXT,
	"amountWithoutTaxes" TEXT,
	"taxesRate" TEXT,
	"taxesCategory" TEXT,
	"ratio" TEXT,
	"comment" TEXT,
	"project" TEXT,
	"category" TEXT,
	"linkedSplit" TEXT,
	"ignoredInBudgets" INTEGER NOT NULL,
	"invoice" TEXT,
	"ignoredInReports" INTEGER NOT NULL,
	"ignoredInAverageBalance" INTEGER NOT NULL,
	"refund" INTEGER NOT NULL,
	"usesAccountOwners" INTEGER NOT NULL
);

CREATE INDEX ICScheduledTransactionSplitTransactionIndex ON ICScheduledTransactionSplit ("transaction");

-- Import

CREATE TABLE ICImportPlugin (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"class" TEXT NOT NULL,
	"list" TEXT NOT NULL,
	"name" TEXT NOT NULL,
	"encoding" TEXT NOT NULL,
	"dateFormat" TEXT NOT NULL,
	"decimalSeparator" TEXT,
	"groupingSeparator" TEXT,
	"accountsMapping" TEXT,
	"transactionsMapping" TEXT,
	"splitsMapping" TEXT,
	"mappedValues" TEXT,
	"categorize" INTEGER NOT NULL,
	"setClearedStatusByDefault" INTEGER NOT NULL,
	"applyRules" INTEGER NOT NULL,
	"reconcile" INTEGER NOT NULL,
	"reconcileUsingName" INTEGER NOT NULL,
	"reconcileUsingDate" INTEGER NOT NULL,
	"numberOfDays" INTEGER NOT NULL,
	"updateDate" INTEGER NOT NULL,
	"updateValueDate" INTEGER NOT NULL,
	"updateName" INTEGER NOT NULL,
	"updateComment" INTEGER NOT NULL,
	"updateStatus" INTEGER NOT NULL,
	"showSettings" INTEGER NOT NULL,
	"showMapping" INTEGER NOT NULL,
	"CSV_separator" TEXT,
	"CSV_firstLine" INTEGER,
	"CSV_hasHeader" INTEGER,
	"CSV_accountNameColumn" TEXT,
	"CSV_accountNumberColumn" TEXT,
	"CSV_splitsCountColumn" TEXT,
	"XML_accountsPath" TEXT,
	"XML_accountNamePath" TEXT,
	"XML_accountNumberPath" TEXT,
	"XML_transactionsPath" TEXT,
	"XML_splitsPath" TEXT,
	"JSON_accountsPath" TEXT,
	"JSON_accountNamePath" TEXT,
	"JSON_accountNumberPath" TEXT,
	"JSON_transactionsPath" TEXT,
	"JSON_splitsPath" TEXT
);

CREATE TABLE ICImportPluginsList (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"pluginsClass" TEXT NOT NULL
);

CREATE TABLE ICImportDataProvider (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"class" TEXT NOT NULL,
	"name" TEXT NOT NULL,
	"pluginClass" TEXT NOT NULL,
	"plugin" TEXT,
	"lastImportDate" TEXT,
	"URL_dataURL" TEXT,
	"OFX_serverURL" TEXT,
	"OFX_login" TEXT,
	"OFX_organization" TEXT,
	"OFX_institutionID" TEXT,
	"OFX_accountType" TEXT,
	"OFX_bankID" TEXT,
	"OFX_brokerID" TEXT,
	"OFX_accountID" TEXT,
	"Linxo_mail" TEXT,
	"Linxo_accountInfos" TEXT
);

-- Export

CREATE TABLE ICExportPlugin (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"class" TEXT NOT NULL,
	"list" TEXT NOT NULL,
	"name" TEXT NOT NULL,
	"encoding" TEXT NOT NULL,
	"dateFormat" TEXT NOT NULL,
	"decimalSeparator" TEXT NOT NULL,
	"groupingSeparator" TEXT NOT NULL,
	"transactionsFields" TEXT,
	"transactionsMapping" TEXT,
	"splitsFields" TEXT,
	"splitsMapping" TEXT,
	"mappedValues" TEXT,
	"showSettings" INTEGER NOT NULL,
	"showMapping" INTEGER NOT NULL,
	"CSV_separator" TEXT,
	"CSV_accountNameColumn" TEXT,
	"CSV_accountNumberColumn" TEXT,
	"XML_accountsPath" TEXT,
	"XML_accountNamePath" TEXT,
	"XML_accountNumberPath" TEXT,
	"XML_transactionsPath" TEXT,
	"XML_splitsPath" TEXT,
	"JSON_accountsPath" TEXT,
	"JSON_accountNamePath" TEXT,
	"JSON_accountNumberPath" TEXT,
	"JSON_transactionsPath" TEXT,
	"JSON_splitsPath" TEXT
);

CREATE TABLE ICExportPluginsList (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"pluginsClass" TEXT NOT NULL
);

CREATE TABLE ICExportDataHandler (
	"sqlID" INTEGER PRIMARY KEY,
	"ID" TEXT UNIQUE NOT NULL,
	"lastModificationDate" TEXT,
	"class" TEXT NOT NULL,
	"name" TEXT NOT NULL,
	"pluginClass" TEXT NOT NULL,
	"plugin" TEXT,
	"URL_dataURL" TEXT
);
